/*********************************************************************
*                    SEGGER Microcontroller GmbH                     *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*            (c) 2014 - 2020 SEGGER Microcontroller GmbH             *
*                                                                    *
*           www.segger.com     Support: support@segger.com           *
*                                                                    *
**********************************************************************
*                                                                    *
* All rights reserved.                                               *
*                                                                    *
* Redistribution and use in source and binary forms, with or         *
* without modification, are permitted provided that the following    *
* conditions are met:                                                *
*                                                                    *
* - Redistributions of source code must retain the above copyright   *
*   notice, this list of conditions and the following disclaimer.    *
*                                                                    *
* - Neither the name of SEGGER Microcontroller GmbH                  *
*   nor the names of its contributors may be used to endorse or      *
*   promote products derived from this software without specific     *
*   prior written permission.                                        *
*                                                                    *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND             *
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,        *
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF           *
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE           *
* DISCLAIMED.                                                        *
* IN NO EVENT SHALL SEGGER Microcontroller GmbH BE LIABLE FOR        *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR           *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  *
* OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;    *
* OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      *
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          *
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE  *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
* DAMAGE.                                                            *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File    : main.c
Purpose : Blinky application for the GigaDevice GD32E231C-START evaluation board

*/

#include <stdio.h>
#include <stdlib.h>

/*********************************************************************
*
*       Defines (fixed)
*
**********************************************************************
*/
#ifndef U8
  #define U8  unsigned char
#endif
#ifndef U16
  #define U16 unsigned short
#endif
#ifndef U32
  #define U32 unsigned long
#endif

#define _GPIOA_BASE_ADDR        (0x48000000) 

#define _GPIOA_CTL             *(volatile U32*)(_GPIOA_BASE_ADDR + 0x0000)
#define _GPIOA_OMODE           *(volatile U32*)(_GPIOA_BASE_ADDR + 0x0004) 
#define _GPIOA_OSPD            *(volatile U32*)(_GPIOA_BASE_ADDR + 0x0008) 
#define _GPIOA_ISTAT           *(volatile U32*)(_GPIOA_BASE_ADDR + 0x0010)
#define _GPIOA_OCTL            *(volatile U32*)(_GPIOA_BASE_ADDR + 0x0014)
#define _GPIOA_BOP             *(volatile U32*)(_GPIOA_BASE_ADDR + 0x0018)
#define _GPIOA_LOCK            *(volatile U32*)(_GPIOA_BASE_ADDR + 0x001C)

#define _RCU_AHBRST            *(volatile U32*)(0x40021028)
#define _RCU_AHBEN             *(volatile U32*)(0x40021014)

#define _USER_LED1              (7) 

/*********************************************************************
*
*       main()
*
*  Function description
*    The application toggles 1 green led (LED1) on the GigaDevice GD32E231C-START
*    evaluation board. For details information regarding this, please refer to:
*    https://wiki.segger.com/GigaDevice_GD32E231C-START
*/
void main(void) {
  U32 v;
  //
  // Enable GPIOA clock, if necessary
  //
  v = _RCU_AHBEN;
  if ((v & (1uL << 17)) == 0) {
    _RCU_AHBEN |= (1uL << 17);
  }
  //
  // Release GPIOA from reset, if necessary
  //
  v = _RCU_AHBRST;
  if (v & (1uL << 17)) {
    _RCU_AHBRST &= ~(1uL << 17);
  }
  //
  // Configure GPIO as output (LOW) 
  //
  _GPIOA_OCTL &= ~(1uL << _USER_LED1);             
  //
  // Set maximum speed 50 MHz
  //
  v = _GPIOA_OSPD;
  v &= ~(3uL << (_USER_LED1*2));
  v |= (3uL << (_USER_LED1*2));                   
  _GPIOA_OSPD = v;
  //
  // Port mode  == Output
  //
  v = _GPIOA_CTL;
  v &= ~(1uL << (_USER_LED1*2));
  v |= (1uL << (_USER_LED1*2));                   
  _GPIOA_CTL = v;
  //
  // Output with push-pull
  //
  _GPIOA_OMODE &= ~(1uL << _USER_LED1);          
  //
  // Turn LED off
  //
  _GPIOA_OCTL &= ~(1uL << _USER_LED1);
  do {
    //
    // Toggle LED
    // 
    v = _GPIOA_OCTL;
    v = (~v) & (1uL << _USER_LED1);
    _GPIOA_OCTL = v;
    //
    // Delay loop
    //
    for (int i = 0; i < 100000; i++) {
      __asm("NOP");
    }
  } while (1);
}

/*************************** End of file ****************************/
